@@ -19,6 +19,7 @@ import sys
 
 import yaml
 from appdirs import user_cache_dir
+from os.path import expanduser
 from cachecontrol import CacheControl
 from cachecontrol.caches.file_cache import FileCache
 # from cachecontrol.heuristics import ExpiresAfter
@@ -76,6 +77,52 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
     if repo.startswith(('http://', 'https://')) and repo.endswith('Chart.yaml'):
         at = 'helm_chart'
 
+    if repo.endswith('.spec'):
+        # repo is specified inside the .spec file
+        # github repo is resolved via %{upstream_github} + %{name}/%{upstream_name}
+        # no upstream_github global means that the spec was not prepared for lastversion
+        # optional: use of spec_tag macros if the source is from GitHub. in edge cases we check
+        # new version via GitHub, but prepared sources are elsewhere
+        with open(repo) as f:
+            name = None
+            upstream_github = None
+            upstream_name = None
+            current_version = None
+            for l in f.readlines():
+                if l.startswith('%global upstream_github'):
+                    upstream_github = l.split(' ')[2].strip()
+                elif l.startswith('%global upstream_name'):
+                    upstream_name = l.split(' ')[2].strip()
+                elif l.startswith('Name:'):
+                    name = l.split('Name:')[1].strip()
+                elif l.startswith('%global upstream_version '):
+                    current_version = l.split(' ')[2].strip()
+                    # influences %spec_tag to use %upstream_version instead of %version
+                    repo_data['module_of'] = True
+                elif l.startswith('Version:') and not current_version:
+                    current_version = l.split('Version:')[1].strip()
+            if not upstream_github:
+                log.critical('%upstream_github macro not found. Please prepare your spec file '
+                             'using instructions: https://github.com/dvershinin/lastversion/wiki/Preparing-RPM-spec-files')
+            if not current_version:
+                log.critical('Did not find neither Version: nor %upstream_version in the spec file')
+                sys.exit(1)
+            try:
+                if current_version != 'x':
+                    repo_data['current_version'] = Version(current_version)
+            except InvalidVersion:
+                log.critical('Failed to parse current version in {}. Tried {}'.format(
+                    repo, current_version))
+                sys.exit(1)
+            if upstream_name:
+                repo_data['name'] = upstream_name
+                repo_data['spec_name'] = '%{upstream_name}'
+            else:
+                repo_data['name'] = name
+                repo_data['spec_name'] = '%{name}'
+            repo = "{}/{}".format(upstream_github, repo_data['name'])
+            log.info('Discovered GitHub repo {} from .spec file'.format(repo))
+
     if (not at or '/' in repo) and at != 'helm_chart':
         # find the right hosting for this repo
         project_holder = HolderFactory.get_instance_for_repo(repo, only=only)
@@ -216,6 +263,76 @@ def parse_version(tag):
     return h.sanitize_version(tag, pre_ok=True)
 
 
+def get_rpm_packager():
+    try:
+        rpmmacros = expanduser("~") + "/.rpmmacros"
+        with open(rpmmacros) as f:
+            for ln in f.readlines():
+                if ln.startswith('%packager'):
+                    return ln.split('%packager')[1].strip()
+    except IOError:
+        log.warning("~/.rpmmacros does not exist. Changelog will not be generated")
+    return None
+
+
+def update_spec(repo, res, sem='minor'):
+    if 'current_version' not in res or res['current_version'] < res['version']:
+        log.info('Updating spec {} with semantic {}'.format(repo, sem))
+        if 'current_version' in res and len(res['version'].release) >= 3:
+            current_major = res['current_version'].release[0]
+            latest_major = res['version'].release[0]
+            current_minor = res['current_version'].release[1]
+            latest_minor = res['version'].release[1]
+            if sem in ['minor', 'patch']:
+                fail_fmt = 'Latest v{} fails semantic {} constraint against current v{}'
+                if latest_major != current_major:
+                    log.warning(
+                        fail_fmt.format(res['version'], sem, res['current_version']))
+                    sys.exit(4)
+                if sem == 'patch' and latest_minor != current_minor:
+                    log.warning(
+                        fail_fmt.format(res['version'], sem, res['current_version']))
+                    sys.exit(4)
+    else:
+        log.info('No newer version than already present in spec file')
+        sys.exit(2)
+    # update %lastversion_tag and %lastversion_dir, Version (?), Release
+    out = []
+    packager = get_rpm_packager()
+    with open(repo) as f:
+        for ln in f.readlines():
+            if ln.startswith('%global lastversion_tag '):
+                out.append('%global lastversion_tag {}'.format(res['spec_tag']))
+            elif ln.startswith('%global lastversion_dir '):
+                out.append('%global lastversion_dir {}-{}'.format(
+                    res['spec_name'], res['spec_tag_no_prefix']))
+            elif ln.startswith('%global upstream_version '):
+                out.append('%global upstream_version {}'.format(res['version']))
+            elif ln.startswith('Version:') and ('module_of' not in res or not res['module_of']):
+                version_tag_regex = r'^Version:(\s+)(\S+)'
+                m = re.match(version_tag_regex, ln)
+                out.append('Version:' + m.group(1) + str(res['version']))
+            elif ln.startswith('%changelog') and packager:
+                import datetime
+                today = datetime.date.today()
+                today = today.strftime('%a %b %d %Y')
+                out.append(ln.rstrip())
+                out.append('* {} {}'.format(today, packager))
+                out.append('- upstream release v{}'.format(res['version']))
+                out.append("\n")
+            elif ln.startswith('Release:'):
+                release_tag_regex = r'^Release:(\s+)(\S+)'
+                m = re.match(release_tag_regex, ln)
+                release = m.group(2)
+                from string import digits
+                release = release.lstrip(digits)
+                out.append('Release:' + m.group(1) + '1' + release)
+            else:
+                out.append(ln.rstrip())
+    with open(repo, "w") as f:
+        f.write("\n".join(out))
+
+
 def main():
     """The entrypoint to CLI app."""
     epilog = None
@@ -224,14 +341,15 @@ def main():
     parser = argparse.ArgumentParser(description='Find the latest software release.',
                                      epilog=epilog,
                                      prog='lastversion')
-    parser.add_argument('action', nargs='?', default='get', help='Special action to run, '
-                                                                 'e.g. download, install, test')
+    parser.add_argument('action', nargs='?', default='get',
+                        help='Special action to run, e.g. download, install, test')
     parser.add_argument('repo', metavar='<repo or URL>',
-                        help='GitHub/GitLab/BitBucket/etc. repository in format owner/name or any URL '
-                             'that belongs to it')
+                        help='Repository in format owner/name or any URL that belongs to it')
     # affects what is considered last release
     parser.add_argument('--pre', dest='pre', action='store_true',
                         help='Include pre-releases in potential versions')
+    parser.add_argument('--sem', dest='sem', choices=['major', 'minor', 'patch'],
+                        help='Semantic version constraint against compared version')
     parser.add_argument('-v', '--verbose', action='count', default=0,
                         help='Will give you an idea of what is happening under the hood, '
                              '-vv to increase verbosity level')
@@ -330,6 +448,15 @@ def main():
         # we can only install assets
         args.format = 'json'
 
+    if args.repo.endswith('.spec'):
+        args.action = 'update-spec'
+        args.format = 'dict'
+
+    if not args.sem:
+        if args.action == 'update-spec':
+            args.sem = 'minor'
+        else:
+            args.sem = 'any'
     # imply source download, unless --assets specified
     # --download is legacy flag to specify download action or name of desired download file
     # --download == None indicates download intent where filename is based on upstream
@@ -358,16 +485,20 @@ def main():
         sys.exit(4)
 
     if res:
-        # download command
+        if args.action == 'update-spec':
+            return update_spec(args.repo, res, sem=args.sem)
         if args.action == 'download':
+            # download command
             if args.format == 'source':
                 # there is only one source, but we need an array
                 res = [res]
+            download_name = None
+            # save with custom filename, if there's one file to download
+            if len(res) == 1:
+                download_name = args.download
             for url in res:
                 log.info("Downloading {} ...".format(url))
-                # there can be many assets, so we do not "rename" them
-                # there can be only one source, so we allow passing custom filename for it
-                download_file(url, args.download if args.format == 'source' else None)
+                download_file(url, download_name)
             sys.exit(0)
 
         if args.action == 'install':
